package com.saa.bondoapp.activity

import android.app.AlertDialog
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.util.Log
import androidx.appcompat.app.AppCompatActivity
import androidx.localbroadcastmanager.content.LocalBroadcastManager

open class BondoTemplateActivity: AppCompatActivity() {

    var popup = false
    val receiverMap = mutableMapOf<String, (Intent?) -> Unit>(
        "com.saa.bondoapp.ACTION_POPUP_INTERNET_DC" to ::internetPopup
    )

    private val broadcastReceiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent?) {
            val action = intent?.action
            if (!receiverMap.containsKey(action)) {
                return
            }
            receiverMap[action]?.invoke(intent)
        }
    }

    private fun internetPopup(intent: Intent?) {
        synchronized(this@BondoTemplateActivity) {
            if (popup) {
                return
            }
            popup = true

            Log.i("ALERT POP", "INTERNET POP UP TRIGGERED")
            val builder = AlertDialog.Builder(this@BondoTemplateActivity)

            builder.setTitle("Alert")
                .setMessage("You're disconnected from the internet.")

            builder.setPositiveButton("OK") { dialog, _ ->
                dialog.dismiss()
                synchronized(this@BondoTemplateActivity) {
                    popup = false
                }
            }

            val dialog = builder.create()
            dialog.show()
        }
    }


    override fun onResume() {
        super.onResume()
        LocalBroadcastManager.getInstance(this)
            .registerReceiver(broadcastReceiver,
                IntentFilter().apply {
                    receiverMap.forEach {
                        addAction(it.key)
                    }
                })
    }

    override fun onPause() {
        super.onPause()
        LocalBroadcastManager.getInstance(this).unregisterReceiver(broadcastReceiver)
    }

}