package com.saa.bondoapp.activity

import android.content.Intent
import android.os.Bundle
import android.util.Log
import android.view.View
import androidx.activity.enableEdgeToEdge
import androidx.lifecycle.lifecycleScope
import com.auth0.android.jwt.JWT
import com.saa.bondoapp.data.db.InMemoryDatabase
import com.saa.bondoapp.data.db.entities.CURRENT_USER_BEARER
import com.saa.bondoapp.data.db.entities.CURRENT_USER_NIM
import com.saa.bondoapp.data.db.entities.Credentials
import com.saa.bondoapp.databinding.ActivityLoginBinding
import com.saa.bondoapp.dto.LoginRequest
import com.saa.bondoapp.service.NetworkConnectivityService
import com.saa.bondoapp.network.VercelApiClient
import com.saa.bondoapp.service.JwtCheckerService
import com.saa.bondoapp.util.GsonS
import com.saa.bondoapp.util.KeyStoreHelper
import com.saa.bondoapp.util.toast
import kotlinx.coroutines.CoroutineExceptionHandler
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import okhttp3.MediaType
import okhttp3.RequestBody
import java.util.Base64

class LoginActivity: BondoTemplateActivity() {

    private lateinit var binding: ActivityLoginBinding

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        enableEdgeToEdge()
        binding = ActivityLoginBinding.inflate(layoutInflater)
        setContentView(binding.root)

        val coroutineExceptionHandler = CoroutineExceptionHandler{_, throwable ->
            throwable.printStackTrace()
        }

        val networkConnectIntent = Intent(applicationContext, NetworkConnectivityService::class.java)
        startService(networkConnectIntent)

        binding.loginBtn.setOnClickListener {
            binding.progressBar.visibility = View.VISIBLE
            val loginRequest = LoginRequest(binding.emailField.text.toString(),
                                                binding.passwordField.text.toString())
            lifecycleScope.launch(Dispatchers.IO + coroutineExceptionHandler) {
                val requestBody = RequestBody.create(MediaType.parse("application/json"), GsonS.instance.toJson(loginRequest))
                try {
                    val response = VercelApiClient.authenticationApiService.login(requestBody).execute()

                    when (response.isSuccessful) {
                        true -> {
                            val intent = Intent(
                                applicationContext,
                                MainTaskActivity::class.java
                            )
                            startActivity(intent)
                            val token: String = response.body()?.token ?: ""
                            val key = KeyStoreHelper.encryptString(token)
                            CURRENT_USER_BEARER = token
                            CURRENT_USER_NIM = JWT(token).getClaim("nim").asLong()!!
                            Log.i("LOGIN", "BERHASIL")
                            Log.i("LOGIN", CURRENT_USER_NIM.toString())


                            val db1 = InMemoryDatabase(this@LoginActivity)
                            db1.credentialsDao().insert(Credentials(
                                extractPayload(token), key, System.currentTimeMillis()))

                            val serviceIntent = Intent(applicationContext, JwtCheckerService::class.java)
                            serviceIntent.putExtra("encryptedToken", key)
                            startService(serviceIntent)
                        }
                        false -> {
                            Log.i("LOGIN", "GAGAL")
                            toast(this@LoginActivity, "Wrong credentials, please do a re-check.")
                        }
                    }
                } catch (e: Exception) {
                    Log.i("LOGIN",
                        "EXCEPTION ${e.message}")
                    toast(this@LoginActivity, "Error while trying to log in.")
                    e.printStackTrace()
                }

                launch(Dispatchers.Main) {
                    binding.progressBar.visibility = View.INVISIBLE
                }
            }
        }
    }

    private fun extractPayload(jwt: String): String {
        val parts = jwt.split(".")
        if (parts.size != 3) {
            return ""
        }

        val payload = parts[1]
        return try {
            String(Base64.getDecoder().decode(payload))
        } catch (e: IllegalArgumentException) {
            ""
        }
    }

}