package com.saa.bondoapp.activity

import android.annotation.SuppressLint
import android.content.Intent
import android.os.Bundle
import android.util.Log
import androidx.activity.OnBackPressedCallback
import androidx.core.os.bundleOf
import androidx.navigation.NavBackStackEntry
import androidx.navigation.NavController
import androidx.navigation.fragment.NavHostFragment
import com.saa.bondoapp.R
import com.saa.bondoapp.databinding.ActivityMainTaskBinding
import com.saa.bondoapp.service.JwtCheckerService


class MainTaskActivity : BondoTemplateActivity() {

    private lateinit var binding: ActivityMainTaskBinding

    private lateinit var navHostFragment: NavHostFragment
    private lateinit var navController: NavController

    private var nominal: Long = -1L

    val labelToIdMap = mapOf(
        "Transactions" to R.id.navigation_transactions,
        "Chart" to R.id.navigation_chart,
        "Settings" to R.id.navigation_settings,
        "Scan" to R.id.navigation_scan
    )

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        binding = ActivityMainTaskBinding.inflate(layoutInflater)
        setContentView(binding.root)

        navHostFragment =
            supportFragmentManager.findFragmentById(R.id.nav_host_fragment) as NavHostFragment
        navController = navHostFragment.navController

        val backCallBack = object : OnBackPressedCallback(true) {
            @SuppressLint("RestrictedApi")
            override fun handleOnBackPressed() {
                var prevItem: NavBackStackEntry? = navController.previousBackStackEntry
                val backstackList = navController.currentBackStack.value
                Log.d("BACKSTACK EVENT: LIST", backstackList.joinToString {
                    it.destination.label.toString()
                })
                Log.d("BACKSTACK EVENT", prevItem!!.destination.label.toString())

                navController.popBackStack()
                setSelectedItem(prevItem.destination.label.toString())
                navController.popBackStack()
            }
        }

        onBackPressedDispatcher.addCallback(this, backCallBack)

        binding.bottomNavigation.setOnItemSelectedListener { item ->
            when (item.itemId) {
                R.id.navigation_transactions -> {
                    binding.fragmentTitle.text = "Transactions"
                    synchronized(this) {
                        if (nominal == -1L) navController.navigate(R.id.TransactionsFragment) else
                            navController.navigate(
                                R.id.TransactionsFragment,
                                bundleOf("nominal" to nominal)
                            )

                        nominal = -1L
                    }
                    true
                }

                R.id.navigation_scan -> {
                    binding.fragmentTitle.text = "Scan"
                    navController.navigate(R.id.ScanFragment)
                    true
                }

                R.id.navigation_chart -> {
                    binding.fragmentTitle.text = "Chart"
                    navController.navigate(R.id.ChartFragment)
                    true
                }

                R.id.navigation_settings -> {
                    binding.fragmentTitle.text = "Settings"
                    navController.navigate(R.id.SettingsFragment)
                    true
                }

                R.id.navigation_tweebs -> {
                    binding.fragmentTitle.text = "Twibbon"
                    navController.navigate(R.id.TweebsFragment)
                    true
                }

                else -> false
            }
        }

        setSelectedItem("Transactions")

        receiverMap["com.saa.bondoapp.RANDOMIZE"] = ::transactionBReceiver
        receiverMap["com.saa.bondoapp.LOGOUT"] = ::logOut
    }

    private fun transactionBReceiver(intent: Intent?) {
        val nominalIntent: Long = intent?.getLongExtra("nominal", 1L) ?: 1L
        synchronized(this) {
            nominal = nominalIntent
        }
        setSelectedItem("Transactions")
    }

    private fun logOut(intent: Intent?) {
        stopService(Intent(this, JwtCheckerService::class.java))

        startActivity(
            Intent(
                applicationContext,
                LoginActivity::class.java
            )
        )
    }

    private fun setSelectedItem(label: String) {
        binding.bottomNavigation.selectedItemId =
            labelToIdMap.getOrDefault(label, R.id.navigation_transactions)
    }

//    @RequiresApi(Build.VERSION_CODES.TIRAMISU)
//    override fun getOnBackInvokedDispatcher(): OnBackInvokedDispatcher {
//        var test = super.getOnBackInvokedDispatcher()
//
//        return test
//    }

}