package com.saa.bondoapp.data.repositories

import android.util.Log
import com.saa.bondoapp.dto.ScanRequest
import com.saa.bondoapp.network.VercelApiClient
import com.saa.bondoapp.data.db.AppDatabase
import com.saa.bondoapp.data.db.entities.CURRENT_USER_BEARER
import com.saa.bondoapp.data.db.entities.CURRENT_USER_NIM
import com.saa.bondoapp.data.db.entities.Transaction
import okhttp3.MediaType
import okhttp3.MultipartBody
import okhttp3.RequestBody
import java.time.LocalDate

class TransactionRepository(
    private val db: AppDatabase
) {
    fun getTransaction(id: Int) = db.getTransactionDao().get(id)
    fun getTransactions() = db.getTransactionDao().getAll()
    suspend fun storeTransaction(transaction: Transaction) = db.getTransactionDao().upsert(transaction)
    suspend fun destroyTransaction(id: Int) = db.getTransactionDao().delete(id)
    suspend fun destroyTransactions() = db.getTransactionDao().deleteAll()
    suspend fun scanTransaction(file: ByteArray): List<Transaction>? {
        val scanRequest = ScanRequest(file)
        val requestBody = RequestBody.create(MediaType.parse("image/*"), scanRequest.file)
        val imagePart = MultipartBody.Part.createFormData("file", "my_image.jpg", requestBody)

        Log.d("DEBUG", CURRENT_USER_BEARER.toString())
        val auth = "Bearer $CURRENT_USER_BEARER"
        val response = VercelApiClient.scanBillApiService.scan(auth, imagePart).execute()

        try {
            when (response.isSuccessful) {
                true -> {
                    val items = response.body()!!.items.items
                    val result = mutableListOf<Transaction>()
                    val currentDate = LocalDate.now()
                    for (item in items) {
                        val transaction = Transaction(
                            currentDate.toString(),
                            item.qty.toString() + " " + item.name,
                            "Pembelian",
                            (item.qty * item.price).toLong(),
                            "Location"
                            )
                        result.add(transaction)
                    }
                    return result
                }
                false -> {
                    Log.d("Unsuccessful", response.errorBody()!!.string())
                    return null
                }
            }
        } catch (e: Exception) {
            Log.d("Debug", e.message!!)
            return null
        }

    }

}