package com.saa.bondoapp.fragment.chart

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import com.patrykandpatrick.vico.core.axis.AxisPosition
import com.patrykandpatrick.vico.core.axis.formatter.AxisValueFormatter
import com.patrykandpatrick.vico.core.axis.horizontal.HorizontalAxis
import com.patrykandpatrick.vico.core.model.CartesianChartModelProducer
import com.patrykandpatrick.vico.core.model.columnSeries
import com.saa.bondoapp.data.db.AppDatabase
import com.saa.bondoapp.data.repositories.TransactionRepository
import com.saa.bondoapp.databinding.FragmentChartBinding
import com.saa.bondoapp.util.toast
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch

class ChartFragment : Fragment(), ChartListener {

    private var _binding: FragmentChartBinding? = null
    private val binding get() = _binding!!
    private val db by lazy { AppDatabase(requireActivity()) }
    private val repository by lazy { TransactionRepository(db) }
    private val producer = CartesianChartModelProducer.build()
    private val categories = listOf("Income", "Expense")
    private val bottomAxisValueFormatter =
        AxisValueFormatter<AxisPosition.Horizontal.Bottom> { x, _, _ -> categories[x.toInt()] }


    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding = FragmentChartBinding.inflate(inflater, container, false);
        val view = binding.root;

        binding.chartView.modelProducer = producer

        with(binding.chartView) {
            (chart?.bottomAxis as HorizontalAxis<AxisPosition.Horizontal.Bottom>).valueFormatter =
                bottomAxisValueFormatter
        }

        return view
    }

    override fun onResume() {
        super.onResume()

        CoroutineScope(Dispatchers.IO).launch {
            // Fetch data from DB and transform
            val transactions = repository.getTransactions()
            var incomeSum: Long = 0
            var expenseSum: Long = 0

            for (t in transactions) {
                if (t.category == "Income") {
                    incomeSum += t.nominal!!
                } else if (t.category == "Expense") {
                    expenseSum += t.nominal!!
                }
            }

            // Update the chart data
            producer.tryRunTransaction {
                columnSeries {
                    series(listOf(1, 2))
                    series(y = listOf(incomeSum, expenseSum))
                }
            }
        }
    }

    override fun onSuccess() {
        toast(requireActivity(), "Success")
    }

    override fun onFailure(message: String) {
        toast(requireActivity(), message)
    }
}