package com.saa.bondoapp.fragment.scan

import android.graphics.Bitmap
import android.location.Location
import android.view.View
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import com.saa.bondoapp.data.db.entities.Transaction
import com.saa.bondoapp.data.repositories.TransactionRepository
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import java.io.ByteArrayOutputStream

class ScanViewModel(
    private val repository: TransactionRepository
) : ViewModel() {
    var image: Bitmap? = null

    var location: MutableLiveData<Location> = MutableLiveData()
    var locality: MutableLiveData<String> = MutableLiveData()

    var locked: Boolean = false

    var _scanListener: ScanListener? = null
    var scanListener: ScanListener
        get() = _scanListener!!
        set(value) {
            _scanListener = value
        }


    fun onProceedButtonClick(view: View) {
        if (locked) {
            return
        }

        if (image == null) {
            scanListener.onFailure("Please capture or pick image!")
            return
        }

        locked = true


        val stream = ByteArrayOutputStream()
        image!!.compress(Bitmap.CompressFormat.PNG, 100, stream)
        val byteArray = stream.toByteArray()

        CoroutineScope(Dispatchers.IO).launch {
            var transactions: List<Transaction>? = null
            try {
                transactions = repository.scanTransaction(byteArray)

                for (transaction in transactions!!) {
                    try {
                        transaction.location_lat = location.value!!.latitude
                        transaction.location_long = location.value!!.longitude
                        transaction.locality = locality.value!!
                    } catch (e: Exception) {
                        continue
                    }
                }

                val countTransactions = transactions.size
                if (countTransactions == 0) {
                    scanListener.onFailure("No transaction found!")
                }
                scanListener.onSuccess(transactions)
            } catch (e: Exception) {
                scanListener.onFailure("Error fetching from server!")
                e.printStackTrace()
            } finally {
                locked = false
            }
        }
    }

}