package com.saa.bondoapp.fragment.scan

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.navigation.NavDirections
import com.google.android.material.bottomsheet.BottomSheetDialogFragment
import com.saa.bondoapp.data.db.AppDatabase
import com.saa.bondoapp.data.db.entities.Transaction
import com.saa.bondoapp.data.repositories.TransactionRepository
import com.saa.bondoapp.databinding.FragmentSubmitTransactionsBinding
import com.saa.bondoapp.fragment.transactions.TransactionsAdapter
import com.saa.bondoapp.util.toast
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext


class SubmitTransactionsFragment() : BottomSheetDialogFragment() {
    var _transactions: List<Transaction> ? = null
    val transactions get() = _transactions!!

    var _binding: FragmentSubmitTransactionsBinding? = null
    val binding get() = _binding!!

    private val transactionsAdapter by lazy { TransactionsAdapter() }


    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        _binding = FragmentSubmitTransactionsBinding.inflate(inflater, container, false);
        val view = binding.root


        fun nothing(nav : NavDirections) {
        }
        transactionsAdapter.navigate = ::nothing


        val db = AppDatabase(requireActivity())
        val repository = TransactionRepository(db)

        binding.scanTransactionSubmit.setOnClickListener {
            CoroutineScope(Dispatchers.IO).launch {

                for (transaction in transactions) {
                    repository.storeTransaction(transaction)
                }

                val countTransactions = transactions.size
                withContext(Dispatchers.Main) {
                    toast(requireActivity(), "Successfully add $countTransactions!")
                }

                dismiss()
            }
        }

        binding.scanTransactionCancel.setOnClickListener {
            toast(requireActivity(), "Scan result discarded!")
            dismiss()
        }

        binding.recyclerViewTransactions.adapter = transactionsAdapter

        return view
    }

    override fun onResume() {
        super.onResume()
        transactionsAdapter.submitList(transactions)
    }

}