package com.saa.bondoapp.fragment.transactions

import android.content.Intent
import android.net.Uri
import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.navigation.NavDirections
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.ListAdapter
import androidx.recyclerview.widget.RecyclerView
import com.saa.bondoapp.data.db.entities.Transaction
import com.saa.bondoapp.databinding.TransactionItemBinding

object TransactionDiffUtil : DiffUtil.ItemCallback<Transaction>() {

    override fun areItemsTheSame(oldItem: Transaction, newItem: Transaction): Boolean {
        return oldItem.id == newItem.id
    }

    override fun areContentsTheSame(oldItem: Transaction, newItem: Transaction): Boolean {
        return oldItem.date == newItem.date
                && oldItem.title == newItem.title
                && oldItem.category == newItem.category
                && oldItem.nominal == newItem.nominal
                && oldItem.locality == newItem.locality
                && oldItem.location_long == newItem.location_long
                && oldItem.location_lat == newItem.location_lat
    }
}

class TransactionsAdapter() :
    ListAdapter<Transaction, TransactionViewHolder>(TransactionDiffUtil) {

    var navigate: ((NavDirections) -> Unit)? = null

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): TransactionViewHolder {
        val binding = TransactionItemBinding.inflate(LayoutInflater.from(parent.context), parent, false)
        return TransactionViewHolder(binding)
    }

    override fun onBindViewHolder(holder: TransactionViewHolder, position: Int) {
        val t = getItem(position)
        holder.bindData(t)
        holder.itemView.setOnClickListener {
            val action = TransactionsFragmentDirections.actionTransactionFragmentToUpsertTransactionFragment(t.id)
            navigate!!(action)
        }
    }
}

class TransactionViewHolder(
    private val binding: TransactionItemBinding
) :
    RecyclerView.ViewHolder(binding.root) {

    fun bindData(t: Transaction) {
        binding.textViewDate.text = t.date
        binding.textViewAmount.text = "IDR ${t.nominal}"
        binding.textViewName.text = t.title
        binding.textViewCategory.text = t.category
        binding.textViewLocation.text = t.locality

        binding.openInMapsButton.setOnClickListener {
            onOpenInMapsClick(t)
        }
    }

    private fun onOpenInMapsClick(t: Transaction) {
        if (t.location_lat != null && t.location_long != null) {
            val mapIntentUri = Uri.parse("geo:${t.location_lat},${t.location_long}")

            val mapIntent = Intent(Intent.ACTION_VIEW, mapIntentUri)

            mapIntent.setPackage("com.google.android.apps.maps")

            binding.root.context.startActivity(mapIntent)
        }
    }
}