package com.saa.bondoapp.fragment.transactions

import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.core.os.bundleOf
import androidx.fragment.app.Fragment
import androidx.lifecycle.ViewModelProvider
import androidx.navigation.NavDirections
import androidx.navigation.fragment.findNavController
import com.saa.bondoapp.R
import com.saa.bondoapp.data.db.AppDatabase
import com.saa.bondoapp.data.repositories.TransactionRepository
import com.saa.bondoapp.databinding.FragmentTransactionsBinding
import com.saa.bondoapp.util.toast
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

class TransactionsFragment : Fragment(), TransactionsListener {

    private var _binding: FragmentTransactionsBinding? = null
    private val binding get() = _binding!!
    private val transactionsAdapter by lazy { TransactionsAdapter() }
    private val db by lazy { AppDatabase(requireActivity()) }
    private val repository by lazy { TransactionRepository(db) }
    private val factory by lazy { TransactionsViewModelFactory(repository) }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        if (arguments !== null) {
            Log.d("arg", arguments.toString())
        }
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding = FragmentTransactionsBinding.inflate(inflater, container, false)
        val view = binding.root

        binding.recyclerViewTransactions.adapter = transactionsAdapter
        fun navFunction(nav: NavDirections) {
            findNavController().navigate(nav)
        }

        transactionsAdapter.navigate = ::navFunction

        val viewModel = ViewModelProvider(this, factory)[TransactionsViewModel::class.java]
        binding.viewmodel = viewModel

        viewModel.transactionsListener = this

        val args = arguments
        if (args?.getLong("nominal") != null) {
            val nominal = args.getLong("nominal")
            findNavController().navigate(R.id.action_TransactionFragment_to_UpsertTransactionFragment,
                bundleOf("nominal" to nominal)
            )
        }

        binding.buttonAddTransaction.setOnClickListener {
            findNavController().navigate(R.id.action_TransactionFragment_to_UpsertTransactionFragment)
        }

        return view
    }

    override fun onResume() {
        super.onResume()
        CoroutineScope(Dispatchers.IO).launch {

            val transactions = repository.getTransactions()

            withContext(Dispatchers.Main) {
                transactionsAdapter.submitList(transactions)
            }
        }
    }

    override fun onSuccess() {
        toast(requireActivity(), "Success")
    }

    override fun onFailure(message: String) {
        toast(requireActivity(), message)
    }
}