package com.saa.bondoapp.fragment.transactions

import android.R.layout
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ArrayAdapter
import androidx.fragment.app.Fragment
import androidx.lifecycle.ViewModelProvider
import androidx.navigation.fragment.findNavController
import androidx.navigation.fragment.navArgs
import com.saa.bondoapp.R
import com.saa.bondoapp.data.db.AppDatabase
import com.saa.bondoapp.data.repositories.TransactionRepository
import com.saa.bondoapp.databinding.FragmentUpsertTransactionBinding
import com.saa.bondoapp.util.LocationContext
import com.saa.bondoapp.util.toast
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import java.time.LocalDate

class UpsertTransactionFragment : Fragment(), UpsertTransactionListener {

    private var _binding: FragmentUpsertTransactionBinding? = null
    private val binding get() = _binding!!

    private var _viewModel: UpsertTransactionViewModel? = null
    private val viewModel get() = _viewModel!!

    lateinit var repository: TransactionRepository

    private lateinit var locationContext: LocationContext

    val args: UpsertTransactionFragmentArgs by navArgs()

    override fun onResume() {
        super.onResume()

        if (arguments !== null) {
            val args = requireArguments()
            if (args.containsKey("nominal")) {
                viewModel.nominal.value = args.getLong("nominal").toString()
            }
        }

        viewModel.id = args.id

        // Add
        if (args.id == -1) {

            binding.transactionUpsertDelete.visibility = View.GONE
            locationContext.updateLocation()
            updateDate()

        } else { // Update

            CoroutineScope(Dispatchers.IO).launch {

                val id = args.id
                val transaction = repository.getTransaction(id)
                withContext(Dispatchers.Main) {
                    viewModel.selectedCategoryIdx.value = viewModel.categoryList?.binarySearch(transaction.category)
                    viewModel.date.value = transaction.date
                    viewModel.locationLong.value = transaction.location_long
                    viewModel.locationLat.value = transaction.location_lat
                    viewModel.locality.value = transaction.locality
                    viewModel.nominal.value = transaction.nominal.toString()
                    viewModel.title.value = transaction.title
                }
            }
            binding.transactionUpsertSubmit.text = "Update"
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        locationContext = LocationContext(this)

        val db = AppDatabase(requireActivity())
        repository = TransactionRepository(db)
        val factory = UpsertTransactionViewModelFactory(repository)

        _viewModel = ViewModelProvider(this, factory).get(UpsertTransactionViewModel::class.java)
        viewModel.locality = locationContext.locality
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding = FragmentUpsertTransactionBinding.inflate(inflater, container, false);
        val view = binding.root;

        binding.viewmodel = viewModel
        binding.lifecycleOwner = this
        viewModel.upsertTransactionListener = this
        _viewModel = viewModel

        val categoryList = listOf("Income", "Expense")
        val adapter = ArrayAdapter(requireContext(), layout.simple_spinner_item, categoryList)
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
        binding.categoryInputEdit.adapter = adapter

        viewModel.categoryList = categoryList

        binding.refreshLocationButton.setOnClickListener {
            locationContext.updateLocation()
        }

        locationContext.location.observe(viewLifecycleOwner) {
            viewModel.locationLat.value = locationContext.location.value?.latitude
            viewModel.locationLong.value = locationContext.location.value?.longitude
        }

        return view
    }

    private fun updateDate() {
        viewModel.date.value = LocalDate.now().toString()
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }

    override fun onSuccess(message: String) {
        toast(requireActivity(), message)
        findNavController().navigate(R.id.action_UpsertTransactionFragment_to_TransactionFragment)

    }

    override fun onFailure(message: String) {
        toast(requireActivity(), message)
    }

}