package com.saa.bondoapp.fragment.transactions

import android.location.Location
import android.os.Build
import android.util.Log
import android.view.View
import android.widget.Adapter
import android.widget.ArrayAdapter
import androidx.annotation.RequiresApi
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.saa.bondoapp.data.db.entities.CURRENT_USER_NIM
import com.saa.bondoapp.data.db.entities.Transaction
import com.saa.bondoapp.data.repositories.TransactionRepository
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import java.time.LocalDate
import java.util.*

class UpsertTransactionViewModel(
    private val repository: TransactionRepository
) : ViewModel() {

    var id:Int? = null
    var date: MutableLiveData<String> = MutableLiveData()
    var title: MutableLiveData<String> = MutableLiveData()
    var nominal: MutableLiveData<String> = MutableLiveData()

    var locationLat: MutableLiveData<Double> = MutableLiveData()
    var locationLong: MutableLiveData<Double> = MutableLiveData()
    var locality: MutableLiveData<String> = MutableLiveData()

    var categoryList: List<String>? = null
    var selectedCategoryIdx: MutableLiveData<Int> = MutableLiveData()

    var upsertTransactionListener: UpsertTransactionListener? = null

    fun onSubmitButtonClick(view: View) {
        val title = title.value
        val date = date.value
        val category = categoryList!![selectedCategoryIdx.value!!]
        val locality = locality.value
        val locationLat = locationLat.value
        val locationLong = locationLong.value
        val nominal = nominal.value?.toLong()

        val logMessage = "Title: $title, Date: $date, Category: $category, Locality: $locality, Latitude: $locationLat, Longitude: $locationLong, Nominal: $nominal"
        Log.d("debug", logMessage)

        if (title == null ||
            nominal == null
        ) {
            upsertTransactionListener?.onFailure("All field must be filled!")
            return
        }

        Log.d("DEBUG", CURRENT_USER_NIM.toString())

        val transaction = Transaction(
            title = title,
            date = date,
            category = category,
            locality = locality,
            location_lat = locationLat,
            location_long = locationLong,
            nominal = nominal,
        )
        if (id != -1) {
            transaction.id = id!!
        }

        CoroutineScope(Dispatchers.IO).launch {
            repository.storeTransaction(transaction)
            withContext(Dispatchers.Main) {
                upsertTransactionListener?.onSuccess(if (id == null)  "Transaction created!" else "Update success!")
            }
        }

    }

    fun onDeleteButtonClick(view: View) {
        CoroutineScope(Dispatchers.IO).launch {
            repository.destroyTransaction(id!!)
            withContext(Dispatchers.Main) {
                upsertTransactionListener?.onSuccess("Transaction deleted!")
            }
        }
    }

}