package com.saa.bondoapp.fragment.tweebs

import android.os.Bundle
import android.util.Log
import androidx.fragment.app.Fragment
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.activity.result.contract.ActivityResultContracts
import androidx.camera.core.CameraSelector
import androidx.camera.core.ImageCapture
import androidx.camera.core.Preview
import androidx.camera.lifecycle.ProcessCameraProvider
import androidx.core.content.ContextCompat
import android.Manifest
import com.saa.bondoapp.databinding.FragmentTweebsBinding
import com.saa.bondoapp.util.toast

class TweebsFragment : Fragment() {

    private var _binding: FragmentTweebsBinding? = null

    private val binding get() = _binding!!
    private var imageCapture: ImageCapture? = null
    private var previewFrozen: Boolean = false
    private var cameraProvider: ProcessCameraProvider? = null

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding = FragmentTweebsBinding.inflate(inflater, container, false)
        val root: View = binding.root

        val permissionLauncher = registerForActivityResult(
            ActivityResultContracts.RequestPermission()
        ) { isGranted ->
            if (isGranted) {
                // Do if the permission is granted
                startCamera()
            } else {
                toast(requireActivity(), "Please grant the camera access!")
            }
        }

        permissionLauncher.launch(Manifest.permission.CAMERA)


        binding.captureTwibbonButton.setOnClickListener {
            freezePreview()
        }

        return root
    }


    private fun startCamera() {
        val cameraProviderFuture = ProcessCameraProvider.getInstance(requireContext())
        cameraProviderFuture.addListener({
            cameraProvider = cameraProviderFuture.get()
            val preview = Preview.Builder().build().also {
                    mPreview ->
                if (!previewFrozen) {
                    mPreview.setSurfaceProvider(binding.previewView.surfaceProvider)
                } else {
                    mPreview.setSurfaceProvider(null)
                }
            }

            imageCapture = ImageCapture.Builder().build()

            val cameraSelector = CameraSelector.DEFAULT_FRONT_CAMERA

            try {
                cameraProvider?.unbindAll()
                cameraProvider?.bindToLifecycle(this, cameraSelector, preview!!)
            } catch (e: Exception) {
                toast(requireActivity(), "Failed to start Camera!")
            }
        }, ContextCompat.getMainExecutor(requireContext()))
    }

    private fun freezePreview() {
        previewFrozen = !previewFrozen
        if (!previewFrozen) {
            binding.captureTwibbonButton.text = "Capture"
        } else {
            binding.captureTwibbonButton.text = "Retake"
        }
        startCamera()
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }

}