package com.saa.bondoapp.fragment.utils

import android.annotation.SuppressLint
import android.location.Geocoder
import android.os.Bundle
import android.text.Editable
import android.text.TextWatcher
import android.util.Log
import androidx.fragment.app.Fragment
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.EditText
import android.widget.TextView
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.saa.bondoapp.R
import com.saa.bondoapp.databinding.FragmentLocationSearchBinding
import java.io.IOException



class LocationSearchFragment : Fragment() {
    private lateinit var editTextSearch: EditText
    private lateinit var recyclerViewSuggestions: RecyclerView

    private var _binding: FragmentLocationSearchBinding? = null
    private val binding get() = _binding!!

    private val suggestions = mutableListOf<String>()
    private lateinit var locationAdapter: LocationAdapter

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        _binding = FragmentLocationSearchBinding.inflate(inflater, container, false);
        val view = binding.root

        editTextSearch = binding.editTextSearch
        recyclerViewSuggestions = binding.recyclerViewSuggestions


        locationAdapter = LocationAdapter(suggestions)
        recyclerViewSuggestions.layoutManager = LinearLayoutManager(requireContext())
        recyclerViewSuggestions.adapter = locationAdapter

        editTextSearch.addTextChangedListener(object : TextWatcher {
            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {}

            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {}

            override fun afterTextChanged(s: Editable?) {
                val query = s.toString()
                if (query.isNotBlank()) {
                    searchLocations(query)
                }
            }
        })

        return view
    }

    private fun searchLocations(query: String) {
        val geocoder = Geocoder(requireContext())
        try {
            val addresses = geocoder.getFromLocationName(query, 10)
            Log.d("DEBUG", addresses.toString())
            suggestions.clear()
            addresses?.forEach { address ->
                suggestions.add(address.getAddressLine(0))
            }
            locationAdapter.notifyDataSetChanged()
        } catch (e: IOException) {
            e.printStackTrace()
        }
    }
}
class LocationAdapter(private val suggestions: List<String>) :
    RecyclerView.Adapter<LocationAdapter.LocationViewHolder>() {

    class LocationViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        val textViewLocation: TextView = itemView.findViewById(android.R.id.text1)
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): LocationViewHolder {
        val view = LayoutInflater.from(parent.context).inflate(android.R.layout.simple_list_item_1, parent, false)
        return LocationViewHolder(view)
    }

    override fun onBindViewHolder(holder: LocationViewHolder, position: Int) {
        holder.textViewLocation.text = suggestions[position]
    }

    override fun getItemCount(): Int {
        return suggestions.size
    }
}

