package com.saa.bondoapp.service

import android.app.Service
import android.content.Intent
import android.os.IBinder
import android.util.Log
import androidx.localbroadcastmanager.content.LocalBroadcastManager
import com.saa.bondoapp.data.db.entities.CURRENT_USER_BEARER
import com.saa.bondoapp.data.db.entities.CURRENT_USER_NIM
import com.saa.bondoapp.network.VercelApiClient
import com.saa.bondoapp.util.KeyStoreHelper
import java.util.concurrent.Executors
import java.util.concurrent.ScheduledExecutorService
import java.util.concurrent.TimeUnit

class JwtCheckerService : Service() {

    private val executor: ScheduledExecutorService = Executors.newSingleThreadScheduledExecutor()
    private var count = 0

    override fun onStartCommand(intent: Intent?, flags: Int, startId: Int): Int {
        executor.scheduleAtFixedRate({
            var token = ""
            try {
                token = intent?.getStringExtra("encryptedToken") ?: ""
                token = KeyStoreHelper.decryptString(token)
            } catch (_ : Exception) { }

            CURRENT_USER_BEARER = token
            val response = VercelApiClient.authenticationApiService
                .checkJwt("Bearer $token").execute()

            when (response.isSuccessful) {
                true -> {
                    CURRENT_USER_NIM = response.body()?.nim!!.toLong()
                    Log.i("JWT CHECKER", "BERHASIL")
                    count +=1
                }
                false -> {
                    Log.i("JWT CHECKER", "GAGAL")
                    LocalBroadcastManager.getInstance(this)
                        .sendBroadcast(Intent("com.saa.bondoapp.LOGOUT"))
                    stopSelf()
                }
            }
        }, 0, 5, TimeUnit.SECONDS)
        return START_NOT_STICKY
    }

    override fun onDestroy() {
        super.onDestroy()
        executor.shutdown()
    }


    override fun onBind(intent: Intent?): IBinder? {
        return null
    }


}