package com.saa.bondoapp.service

import android.app.Service
import android.content.Context
import android.content.Intent
import android.net.ConnectivityManager
import android.net.Network
import android.net.NetworkCapabilities
import android.net.NetworkRequest
import android.os.IBinder
import android.util.Log
import androidx.localbroadcastmanager.content.LocalBroadcastManager

class NetworkConnectivityService : Service() {

    companion object {
        private val lock = Any()
        private var created = false
    }

    private lateinit var sysConnectivityManager: ConnectivityManager

    private val callback = object : ConnectivityManager.NetworkCallback() {
        override fun onLost(network: Network) {
            Log.i("NETWORK", "LOST CONN")
            LocalBroadcastManager.getInstance(this@NetworkConnectivityService)
                .sendBroadcast(Intent("com.saa.bondoapp.ACTION_POPUP_INTERNET_DC"))
        }
    }

    override fun onStartCommand(intent: Intent?, flags: Int, startId: Int): Int {
        synchronized(lock) {
            if (created) {
                return START_NOT_STICKY
            }

            Log.i("NETWORK CONNECTIVITY", "starting service")
            sysConnectivityManager =
                applicationContext.getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager

            val networkRequest = NetworkRequest.Builder()
                .addCapability(NetworkCapabilities.NET_CAPABILITY_INTERNET)
                .build()
            sysConnectivityManager.registerNetworkCallback(networkRequest, callback)
            created = true
            return START_NOT_STICKY
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        sysConnectivityManager.unregisterNetworkCallback(callback)
        Log.i("MEM_MANAGEMENT", "Activity manager unregistered")
        created = false
    }

    override fun onBind(intent: Intent?): IBinder? {
        return null
    }


}