package com.saa.bondoapp.util

import android.security.keystore.KeyGenParameterSpec
import android.security.keystore.KeyProperties
import android.util.Base64
import java.security.KeyPairGenerator
import java.security.KeyStore
import javax.crypto.Cipher

object KeyStoreHelper {

    private val keyStore = KeyStore.getInstance("AndroidKeyStore")
    private const val KEY_STORE_ALIAS = "BONDO_APP_ALIAS"
    private val cipher: Cipher = Cipher.getInstance("RSA/ECB/PKCS1Padding")

    private val privateKey = lazy {
        keyStore.getKey(KEY_STORE_ALIAS, null)
    }

    private val publicKey = lazy {
        keyStore.getCertificate(KEY_STORE_ALIAS)
    }

    init {
        keyStore.load(null)
        if (!keyStore.containsAlias(KEY_STORE_ALIAS)) {
            generateKeyPair()
        }
    }

    private fun generateKeyPair() {
        val keyPairGenerator =
            KeyPairGenerator.getInstance(KeyProperties.KEY_ALGORITHM_RSA, "AndroidKeyStore")
        keyPairGenerator.initialize(
            KeyGenParameterSpec.Builder(
                KEY_STORE_ALIAS,
                KeyProperties.PURPOSE_ENCRYPT or KeyProperties.PURPOSE_DECRYPT)
                .setBlockModes(KeyProperties.BLOCK_MODE_ECB)
                .setEncryptionPaddings(KeyProperties.ENCRYPTION_PADDING_RSA_PKCS1)
                .build()
        )
        keyPairGenerator.generateKeyPair()
    }

    fun encryptString(value: String): String {
        cipher.init(Cipher.ENCRYPT_MODE, publicKey.value)
        val encryptedBytes : ByteArray = cipher.doFinal(value.toByteArray())
        return Base64.encodeToString(encryptedBytes, Base64.DEFAULT)
    }

    fun decryptString(value: String): String {
        cipher.init(Cipher.DECRYPT_MODE, privateKey.value)
        val decryptedBytes = cipher.doFinal(Base64.decode(value, Base64.DEFAULT))
        return String(decryptedBytes)
    }


}