package com.saa.bondoapp.util

import android.Manifest
import android.content.pm.PackageManager
import android.location.Geocoder
import android.location.Location
import android.util.Log
import androidx.activity.result.contract.ActivityResultContracts
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import androidx.lifecycle.MutableLiveData
import com.google.android.gms.location.FusedLocationProviderClient
import com.google.android.gms.location.LocationServices
import java.util.Locale

open class LocationContext(val fragment: Fragment) {
    var fusedLocationProviderClient : FusedLocationProviderClient
    open var locality: MutableLiveData<String> = MutableLiveData()
    open var location: MutableLiveData<Location> = MutableLiveData()
    var geocoder:Geocoder

    private val DEFAULT_LAT = 6.8773
    private val DEFAULT_LONG = 107.6174
    private val DEFAULT_LOCALITY = "Dago"


    init {
        askPermission()
        fusedLocationProviderClient = LocationServices.getFusedLocationProviderClient(fragment.requireActivity())
        geocoder = Geocoder(fragment.requireContext(), Locale.getDefault())
    }
    open fun updateLocation(): Boolean {
        if (ContextCompat.checkSelfPermission(
                fragment.requireContext(),
                android.Manifest.permission.ACCESS_FINE_LOCATION
            ) == PackageManager.PERMISSION_GRANTED
        ) {
            fusedLocationProviderClient.lastLocation
                .addOnSuccessListener { location ->
                    location?.let {
                        this.location.value = location
                        this.locality.value = getLocality(this.location.value!!)
                    } ?: run {
                        setDefault()
                        Log.d("DEBUG", "Last location is null!")
                    }
                }.addOnFailureListener {
                    setDefault()
                }
            return true
        } else {
            toast(fragment.requireActivity(), "Please grant the location access!")
            return false
        }
    }

    fun getLocality(location: Location): String? {
        return try {
            val addresses = geocoder.getFromLocation(location.latitude, location.longitude, 1)
            addresses!![0].locality.toString()
        } catch (e: Exception) {
            toast(fragment.requireActivity(), "Please make sure you have good internet connection!")
            null
        }
    }

    fun askPermission() {
        val permissionLauncher = fragment.registerForActivityResult(
            ActivityResultContracts.RequestPermission()
        ) { isGranted ->
            if (isGranted) {
                //
            } else {
                //
            }
        }
        permissionLauncher.launch(Manifest.permission.ACCESS_FINE_LOCATION)
    }

    private fun setDefault() {
        val defaultLocation = Location("DEFAULT")
        defaultLocation.latitude = DEFAULT_LAT
        defaultLocation.longitude = DEFAULT_LONG
        this.location.value = defaultLocation
        this.locality.value = DEFAULT_LOCALITY
    }


    // TODO: handle empty cache
}